#! /usr/bin/env python
# $Header: /us/jack/pylib/zsi/ZSI/TCtimes.py,v 1.2 2001/08/17 17:54:21 rsalz Exp $
'''Typecodes for dates and times.
'''

from ZSI import _copyright, _floattypes, _inttypes, EvaluateException
from ZSI.TC import TypeCode
import operator, re, time

_niltime = [
    0, 0, 0,	# year month day
    0, 0, 0,	# hour minute second
    0, 0, 0	# weekday, julian day, dst flag
]

def _dict_to_tuple(d):
    '''Convert a dictionary to a time tuple.  Depends on key values in the
    regexp pattern!
    '''
    retval = _niltime[:]
    for k,i in ( ('Y', 0), ('M', 1), ('D', 2), ('h', 3), ('m', 4), ):
	retval[i] = int(d[k])
    v = d.get('s', None)
    if v: retval[5] = int(float(v))
    v = d.get('tz', None)
    if v and v != 'Z':
	h,m = map(int, v.split(':'))
	if h < 0:
	    retval[3] += abs(h)
	    retval[4] += m
	else:
	    retval[3] -= abs(h)
	    retval[4] -= m
    if d.get('neg', 0):
	retval[0:5] = map(operator.__neg__, retval[0:5])
    return retval

class Duration(TypeCode):
    '''Time duration.
    '''

    parselist = [ (None,'duration') ]
    lex_pattern = re.compile(r'(?P<neg>-?)' \
			r'(?P<Y>\d+Y)?' r'(?P<M>\d+M)?' r'(?P<D>\d+D)?' \
			r'(?P<T>T?)' \
			r'(?P<h>\d+H)?' r'(?P<m>\d+M)?' r'(?P<s>\d*(\.\d+)?S)?')

    def __init__(self, pname=None, **kw):
	TypeCode.__init__(self, pname, **kw)

    def parse(self, elt, ps):
	self.checkname(elt, ps)
	elt = self.SimpleHREF(elt, ps, 'duration')
	if not elt: return None
	if self.nilled(elt, ps): return None
	v = self.simple_value(elt, ps).strip()
	m = Duration.lex_pattern.match(v)
	if m == None:
	    raise EvaluateException('Illegal duration', ps.Backtrace(elt))
	d = m.groupdict()
	if d['T'] and (d['h'] == None and d['m'] == None and d['s'] == None):
	    raise EvaluateException('Duration has T without time',
		ps.Backtrace(elt))
	try:
	    retval = _dict_to_tuple(d)
	except ValueError, e:
	    raise EvaluateException(str(e), ps.Backtrace(elt))
	return retval

    def serialize(self, sw, pyobj, **kw):
	if 1 in map(lambda x: x < 0, pyobj):
	    pyobj = map(abs, pyobj)
	    neg = '-'
	else:
	    neg = ''
	n = kw.get('name', self.oname) or ('E%x' % id(pyobj))
	val = '%sPY%dM%dD%dTh%dm%ds%d' % \
	    ( neg, pyobj[0], pyobj[1], pyobj[2], pyobj[3], pyobj[4], pyobj[5])
	if kw.get('typed', self.typed):
	    tstr = ' xsi:type="xsd:duration"'
	else:
	    tstr = ''
	print >>sw, '<%s%s%s>%s</%s>' % \
		(n, kw.get('attrtext', ''), tstr, val, n)

class Gregorian(TypeCode):
    '''Gregorian times.
    '''

    def __init__(self, pname=None, **kw):
	TypeCode.__init__(self, pname, **kw)

    def parse(self, elt, ps):
	self.checkname(elt, ps)
	elt = self.SimpleHREF(elt, ps, 'Gregorian')
	if not elt: return None
	if self.nilled(elt, ps): return None
	v = self.simple_value(elt, ps).strip()
	m = self.lex_pattern.match(v)
	if not m:
	    raise EvaluateException('Bad Gregorian', ps.Backtrace(elt))
	try:
	    retval = _dict_to_tuple(m.groupdict())
	except ValueError, e:
	    raise EvaluateException(str(e), ps.Backtrace(elt))
	return retval

    def serialize(self, sw, pyobj, **kw):
	if type(pyobj) in _floattypes or type(pyobj) in _inttypes:
	    pyobj = time.gmtime(pyobj)
	if 1 in map(lambda x: x < 0, pyobj):
	    val = '-' + time.strftime(self.format, map(abs, pyobj))
	else:
	    val = time.strftime(self.format, pyobj)
	n = kw.get('name', self.oname) or ('E%x' % id(pyobj))
	if kw.get('typed', self.typed):
	    tstr = ' xsi:type="xsd:%s"' % self.tag
	else:
	    tstr = ''
	print >>sw, '<%s%s%s>%s</%s>' % \
		    (n, kw.get('attrtext', ''), tstr, val, n),

class gDateTime(Gregorian):
    '''A date and time.
    '''
    parselist = [ (None,'dateTime') ]
    lex_pattern = re.compile(r'(?P<neg>-?)' \
			'(?P<Y>\d{4,})-' r'(?P<M>\d\d)-' r'(?P<D>\d\d)' 'T' \
			r'(?P<h>\d\d):' r'(?P<m>\d\d):' r'(?P<s>\d*(\.\d+)?)' \
			r'(?P<tz>(Z|([-+]?\d\d:\d\d))?)')
    tag, format = 'dateTime', '%Y-%m-%dT%H:%M:%SZ'

    def __init__(self, pname=None, **kw):
	Gregorian.__init__(self, pname, **kw)

class gDate(Gregorian):
    '''A date.
    '''
    parselist = [ (None,'date') ]
    lex_pattern = re.compile(r'(?P<neg>-?)' \
			'(?P<Y>\d{4,})-' r'(?P<M>\d\d)-' r'(?P<D>\d\d)' \
			r'(?P<tz>Z|([-+]?\d\d:\d\d))?')
    tag, format = 'date', '%Y-%m-%dT%H:%M:%SZ'

    def __init__(self, pname=None, **kw):
	Gregorian.__init__(self, pname, **kw)

class gYearMonth(Gregorian):
    '''A date.
    '''
    parselist = [ (None,'gYearMonth') ]
    lex_pattern = re.compile(r'(?P<neg>-?)' \
			'(?P<Y>\d{4,})-' r'(?P<M>\d\d)' \
			r'(?P<tz>Z|([-+]?\d\d:\d\d))?')
    tag, format = 'gYearMonth', '%Y-%mZ'

    def __init__(self, pname=None, **kw):
	Gregorian.__init__(self, pname, **kw)

class gYear(Gregorian):
    '''A date.
    '''
    parselist = [ (None,'gYear') ]
    lex_pattern = re.compile(r'(?P<neg>-?)' \
			'(?P<Y>\d{4,})' \
			r'(?P<tz>Z|([-+]?\d\d:\d\d))?')
    tag, format = 'gYear', '%YZ'

    def __init__(self, pname=None, **kw):
	Gregorian.__init__(self, pname, **kw)

class gMonthDay(Gregorian):
    '''A gMonthDay.
    '''
    parselist = [ (None,'gMonthDay') ]
    lex_pattern = re.compile(r'(?P<neg>-?)' \
			r'--(?P<M>\d\d)-' r'(?P<D>\d\d)' \
			r'(?P<tz>Z|([-+]?\d\d:\d\d))?')
    tag, format = 'gMonthDay', '%m-%d'

    def __init__(self, pname=None, **kw):
	Gregorian.__init__(self, pname, **kw)

class gDay(Gregorian):
    '''A gDay.
    '''
    parselist = [ (None,'gDay') ]
    lex_pattern = re.compile(r'(?P<neg>-?)' \
			r'---(?P<D>\d\d)' \
			r'(?P<tz>Z|([-+]?\d\d:\d\d))?')
    tag, format = 'gDay', '%dZ'

    def __init__(self, pname=None, **kw):
	Gregorian.__init__(self, pname, **kw)

class gTime(Gregorian):
    '''A time.
    '''
    parselist = [ (None,'time') ]
    lex_pattern = re.compile(r'(?P<neg>-?)' \
			r'(?P<h>\d\d):' r'(?P<m>\d\d):' r'(?P<s>\d*(\.\d+)?)' \
			r'(?P<tz>Z|([-+]?\d\d:\d\d))?')
    tag, format = 'time', '%H:%M:%SZ'

    def __init__(self, pname=None, **kw):
	Gregorian.__init__(self, pname, **kw)

if __name__ == '__main__': print _copyright
